@@ -23,9 +23,10 @@
 #include <config.h>
 #include "bg-colors-source.h"
 
-#include "gnome-wp-item.h"
+#include "cc-background-item.h"
 
 #include <glib/gi18n-lib.h>
+#include <gdesktop-enums.h>
 
 G_DEFINE_TYPE (BgColorsSource, bg_colors_source, BG_TYPE_SOURCE)
 
@@ -40,51 +41,79 @@ bg_colors_source_class_init (BgColorsSourceClass *klass)
 struct {
 	const char *name;
 	GDesktopBackgroundShading type;
+	int orientation;
 } items[] = {
-	{ N_("Horizontal Gradient"), G_DESKTOP_BACKGROUND_SHADING_HORIZONTAL },
-	{ N_("Vertical Gradient"), G_DESKTOP_BACKGROUND_SHADING_VERTICAL },
-	{ N_("Solid Color"), G_DESKTOP_BACKGROUND_SHADING_SOLID },
+	{ N_("Horizontal Gradient"), G_DESKTOP_BACKGROUND_SHADING_HORIZONTAL, GTK_ORIENTATION_HORIZONTAL },
+	{ N_("Vertical Gradient"), G_DESKTOP_BACKGROUND_SHADING_VERTICAL, GTK_ORIENTATION_VERTICAL },
+	{ N_("Solid Color"), G_DESKTOP_BACKGROUND_SHADING_SOLID, -1 },
 };
 
 #define PCOLOR "#023c88"
 #define SCOLOR "#5789ca"
 
+static GEmblem *
+get_arrow_icon (GtkOrientation orientation)
+{
+  GIcon *themed;
+  GEmblem *emblem;
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    themed = g_themed_icon_new ("go-next-symbolic");
+  else
+    themed = g_themed_icon_new ("go-down-symbolic");
+  emblem = g_emblem_new_with_origin (themed, G_EMBLEM_ORIGIN_DEVICE);
+  g_object_unref (themed);
+  return emblem;
+}
+
 static void
 bg_colors_source_init (BgColorsSource *self)
 {
   GnomeDesktopThumbnailFactory *thumb_factory;
   guint i;
   GtkListStore *store;
-  GdkColor pcolor, scolor;
 
   store = bg_source_get_liststore (BG_SOURCE (self));
 
   thumb_factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 
-  gdk_color_parse (PCOLOR, &pcolor);
-  gdk_color_parse (SCOLOR, &scolor);
-
   for (i = 0; i < G_N_ELEMENTS (items); i++)
     {
-      GnomeWPItem *item;
+      CcBackgroundItemFlags flags;
+      CcBackgroundItem *item;
       GIcon *pixbuf;
 
-      item = g_new0 (GnomeWPItem, 1);
+      item = cc_background_item_new (NULL);
+      flags = CC_BACKGROUND_ITEM_HAS_PCOLOR |
+	      CC_BACKGROUND_ITEM_HAS_SCOLOR |
+	      CC_BACKGROUND_ITEM_HAS_SHADING |
+	      CC_BACKGROUND_ITEM_HAS_PLACEMENT |
+	      CC_BACKGROUND_ITEM_HAS_URI;
+      /* It does have a URI, it's "none" */
 
-      item->filename = g_strdup ("(none)");
-      item->name = g_strdup (_(items[i].name));
-
-      item->pcolor = gdk_color_copy (&pcolor);
-      item->scolor = gdk_color_copy (&scolor);
-
-      item->shade_type = items[i].type;
-
-      gnome_wp_item_ensure_gnome_bg (item);
+      g_object_set (G_OBJECT (item),
+		    "name", _(items[i].name),
+		    "primary-color", PCOLOR,
+		    "secondary-color", SCOLOR,
+		    "shading", items[i].type,
+		    "placement", G_DESKTOP_BACKGROUND_STYLE_NONE,
+		    "flags", flags,
+		    NULL);
 
       /* insert the item into the liststore */
-      pixbuf = gnome_wp_item_get_thumbnail (item,
-                                            thumb_factory,
-                                            THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT);
+      pixbuf = cc_background_item_get_thumbnail (item,
+						 thumb_factory,
+						 THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT);
+      if (items[i].orientation != -1)
+        {
+          GEmblem *emblem;
+          GIcon *icon;
+
+	  emblem = get_arrow_icon (items[i].orientation);
+	  icon = g_emblemed_icon_new (G_ICON (pixbuf), emblem);
+	  g_object_unref (emblem);
+	  g_object_unref (pixbuf);
+	  pixbuf = icon;
+	}
       gtk_list_store_insert_with_values (store, NULL, 0,
                                          0, pixbuf,
                                          1, item,
