@@ -35,7 +35,7 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
-#include <gsettings-desktop-schemas/gdesktop-enums.h>
+#include <gdesktop-enums.h>
 
 #include "xrandr-capplet.h"
 
@@ -251,7 +251,6 @@ add_key (GtkWidget *widget,
   GtkComboBox *box = GTK_COMBO_BOX (widget);
   GtkTreeModel *model = gtk_combo_box_get_model (box);
   GtkListStore *store = GTK_LIST_STORE (model);
-  gboolean retval;
 
   info.text = text;
   info.found = FALSE;
@@ -270,11 +269,6 @@ add_key (GtkWidget *widget,
                                          5, rotation,
                                          -1);
 
-      retval = TRUE;
-    }
-  else
-    {
-      retval = FALSE;
     }
 }
 
@@ -682,8 +676,6 @@ rebuild_resolution_combo (App *app)
 static void
 rebuild_gui (App *app)
 {
-  gboolean sensitive;
-
   /* We would break spectacularly if we recursed, so
    * just assert if that happens
    */
@@ -691,22 +683,12 @@ rebuild_gui (App *app)
 
   app->ignore_gui_changes = TRUE;
 
-  sensitive = app->current_output ? TRUE : FALSE;
-
-#if 0
-  g_debug ("rebuild gui, is on: %d", app->current_output->on);
-#endif
-
   rebuild_mirror_screens (app);
   rebuild_current_monitor_label (app);
   rebuild_on_off_radios (app);
   rebuild_resolution_combo (app);
   rebuild_rotation_combo (app);
 
-#if 0
-  g_debug ("sensitive: %d, on: %d", sensitive, gnome_rr_output_info_is_active (app->current_output));
-#endif
-
   app->ignore_gui_changes = FALSE;
 }
 
@@ -1053,10 +1035,23 @@ on_clone_changed (GtkWidget *box, gpointer data)
 }
 
 static void
-get_geometry (GnomeRROutputInfo *output, int *w, int *h)
+apply_rotation_to_geometry (GnomeRROutputInfo *output, int *w, int *h)
 {
   GnomeRRRotation rotation;
 
+  rotation = gnome_rr_output_info_get_rotation (output);
+  if ((rotation & GNOME_RR_ROTATION_90) || (rotation & GNOME_RR_ROTATION_270))
+    {
+      int tmp;
+      tmp = *h;
+      *h = *w;
+      *w = tmp;
+    }
+}
+
+static void
+get_geometry (GnomeRROutputInfo *output, int *w, int *h)
+{
   if (gnome_rr_output_info_is_active (output))
     {
       gnome_rr_output_info_get_geometry (output, NULL, NULL, w, h);
@@ -1067,14 +1062,7 @@ get_geometry (GnomeRROutputInfo *output, int *w, int *h)
       *w = gnome_rr_output_info_get_preferred_width (output);
     }
 
-  rotation = gnome_rr_output_info_get_rotation (output);
-  if ((rotation & GNOME_RR_ROTATION_90) || (rotation & GNOME_RR_ROTATION_270))
-    {
-      int tmp;
-      tmp = *h;
-      *h = *w;
-      *w = tmp;
-    }
+  apply_rotation_to_geometry (output, w, h);
 }
 
 #define SPACE 15
@@ -1183,6 +1171,8 @@ list_edges_for_output (GnomeRROutputInfo *output, GArray *edges)
 
   gnome_rr_output_info_get_geometry (output, &x, &y, &w, &h);
 
+  apply_rotation_to_geometry (output, &w, &h);
+
   /* Top, Bottom, Left, Right */
   add_edge (output, x, y, x + w, y, edges);
   add_edge (output, x, y + h, x + w, y + h, edges);
@@ -1380,6 +1370,8 @@ static void
 get_output_rect (GnomeRROutputInfo *output, GdkRectangle *rect)
 {
   gnome_rr_output_info_get_geometry (output, &rect->x, &rect->y, &rect->width, &rect->height);
+
+  apply_rotation_to_geometry (output, &rect->width, &rect->height);
 }
 
 static gboolean
@@ -1916,7 +1908,6 @@ paint_output (App *app, cairo_t *cr, int i)
 
   cairo_save (cr);
 
-  layout = get_display_name (app, output);
   layout_set_font (layout, "Sans 10");
   pango_layout_get_pixel_extents (layout, &ink_extent, &log_extent);
 
