@@ -47,7 +47,7 @@
 #include "um-editable-button.h"
 #include "um-editable-entry.h"
 #include "um-editable-combo.h"
-#include "um-lockbutton.h"
+#include "libgnome-control-center/cc-lockbutton.h"
 
 #include "um-account-dialog.h"
 #include "um-language-dialog.h"
@@ -463,6 +463,32 @@ delete_user (GtkButton *button, UmUserPanelPrivate *d)
 
 }
 
+static const gchar *
+get_invisible_text (void)
+{
+   GtkWidget *entry;
+   gunichar invisible_char;
+   static gchar invisible_text[40];
+   gchar *p;
+   gint i;
+
+   entry = gtk_entry_new ();
+   invisible_char = gtk_entry_get_invisible_char (GTK_ENTRY (entry));
+   if (invisible_char == 0)
+     invisible_char = 0x2022;
+
+   g_object_ref_sink (entry);
+   g_object_unref (entry);
+
+   /* five bullets */
+   p = invisible_text;
+   for (i = 0; i < 5; i++)
+     p += g_unichar_to_utf8 (invisible_char, p);
+   *p = 0;
+
+   return invisible_text;
+}
+
 static const gchar *
 get_password_mode_text (UmUser *user)
 {
@@ -474,8 +500,7 @@ get_password_mode_text (UmUser *user)
         else {
                 switch (um_user_get_password_mode (user)) {
                 case UM_PASSWORD_MODE_REGULAR:
-                        /* five bullets */
-                        text = "\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2";
+                        text = get_invisible_text ();
                         break;
                 case UM_PASSWORD_MODE_SET_AT_LOGIN:
                         text = C_("Password mode", "To be set at next login");
@@ -1169,7 +1194,7 @@ setup_main_window (UmUserPanelPrivate *d)
         d->permission = (GPermission *)polkit_permission_new_sync ("org.freedesktop.accounts.user-administration", NULL, NULL, NULL);
         if (d->permission != NULL) {
                 /* accounts service not available? */
-                button = um_lock_button_new (d->permission);
+                button = cc_lock_button_new (d->permission);
                 gtk_widget_set_margin_top (button, 12);
                 gtk_widget_show (button);
                 box = get_widget (d, "accounts-vbox");
